#include <Adafruit_GFX.h>
#include <SPI.h>
#include <Wire.h>
#include <Adafruit_ILI9341.h>
#include <Adafruit_STMPE610.h>

// This is calibration data for the raw touch data to the screen coordinates
#define TS_MINX 150
#define TS_MINY 130
#define TS_MAXX 3800
#define TS_MAXY 4000

#define STMPE_CS 8
Adafruit_STMPE610 ts = Adafruit_STMPE610(STMPE_CS);
#define TFT_CS 10
#define TFT_DC 9
Adafruit_ILI9341 tft = Adafruit_ILI9341(TFT_CS, TFT_DC, -1, false);

#define BOXSIZE 33
int oldcolor, currentcolor;
boolean RecordOn = false;
int x = 0;

uint8_t uCurrentScreen = 0;
enum {
  SCREEN_MAIN=0, SCREEN_SECOND};

//---------------------------------------------------------
// Some hacks for switching SPI registers
enum {
  SPIR_TFT=0, SPIR_TS, SPIR_CNT};
extern void SaveSPIRegs(uint8_t spir);
extern void RestoreSPIRegs(uint8_t spir);
//-----------------------------------------------------------

void setup(void)
{
  delay(2000);
  Serial.begin(9600);
  PrintPinsConfig();
  tft.begin();
  PrintPinsConfig();
  SaveSPIRegs(SPIR_TFT);
  if (!ts.begin()) { 

    Serial.println("Unable to start touchscreen.");
    delay(2000);
  } 
  else { 

    Serial.println("Touchscreen started."); 
    delay(2000);
  }
  PrintPinsConfig();
  SaveSPIRegs(SPIR_TS);

  RestoreSPIRegs(SPIR_TFT);
  PrintPinsConfig();
  tft.setRotation(3);
  SetDisplayMainScreen();


}

void loop()
{
  // See if there's any  touch data for us
  //if (ts.bufferEmpty()) {
  // return;
  //}

  // You can also wait for a touch
  RestoreSPIRegs(SPIR_TS);
  if (! ts.touched()) {
    return;
  }
  switch (uCurrentScreen) {
  case SCREEN_MAIN: 
    ProcessTouchMainScreen();
    break;
  case SCREEN_SECOND: 
    ProcessTouchSecondScreen();
    break;
  }
}  

//---------------------------------------------------------
// Some hacks for switching SPI registers
uint8_t   g_icurSPIR = 0xff;  // Set to something that won't be
uint32_t  g_aluSPI0_CTAR0[SPIR_CNT];
uint32_t  g_alsuSPI0_PIN10_CONFIG[SPIR_CNT];  // pin 10 CS gets mucked with...
uint32_t  g_aluSPI0_MCR[SPIR_CNT];

void SaveSPIRegs(uint8_t spir) {
  g_aluSPI0_MCR[spir] = SPI0_MCR;
  g_aluSPI0_CTAR0[spir] = SPI0_CTAR0;
  g_alsuSPI0_PIN10_CONFIG[spir] = CORE_PIN10_CONFIG;
}

void RestoreSPIRegs(uint8_t spir) {
  if (g_icurSPIR != spir) {  
    // Need to wait until all outputs are done... 
    // Wait until hardware queue is empty...
    uint16_t wTimeout = 0xffff;
    while (((SPI0.SR) & (15 << 12)) && (--wTimeout)) ; // wait until empty

    // May want to wait until actual output is done? also may need to turn off SPI before change?
    // But for now simply try restoring the register(s)
    SPI0.MCR = SPI_MCR_MDIS | SPI_MCR_HALT;
    SPI0_CTAR0 = g_aluSPI0_CTAR0[spir];
    SPI0_MCR = g_aluSPI0_MCR[spir];
    CORE_PIN10_CONFIG = g_alsuSPI0_PIN10_CONFIG[spir];
    g_icurSPIR = spir;
  }
}
//-----------------------------------------------------------

void PrintPinsConfig(void) {
  Serial.print(CORE_PIN8_CONFIG, HEX);
  Serial.print(" ");
  Serial.print(CORE_PIN9_CONFIG, HEX);
  Serial.print(" ");
  Serial.print(CORE_PIN10_CONFIG, HEX);
  Serial.print(" ");
  Serial.print(CORE_PIN11_CONFIG, HEX);
  Serial.print(" ");
  Serial.print(CORE_PIN12_CONFIG, HEX);
  Serial.print(" ");
  Serial.print(CORE_PIN13_CONFIG, HEX);
  Serial.println(" ");
  //  Serial.print(SPI0.MCR, HEX);
  //  Serial.print(" ");
  //  uint32_t ulSR = SPI0.SR;
  //  Serial.println(SPI0.SR, HEX);
}

//-------------------------------------------------------------------------
//
//-------------------------------------------------------------------------

void SetDisplayMainScreen(){

  int i;
 
  tft.fillScreen(ILI9341_BLACK);
  Serial.println("BLACK");
//  delay(2000);


tft. setTextColor(ILI9341_BLUE);
//setTextColor(uint16_t color, uint16_t backgroundcolor);
tft.setTextSize(3);

  for(i=0; i<=7; i++)
  {
   tft.fillRoundRect(((i*6)+7)+(i*BOXSIZE), 7, BOXSIZE, BOXSIZE, 7, ILI9341_WHITE);
   tft.setCursor(((i*6)+16)+(i*BOXSIZE), 13);
   tft.print(i);
  // tft.drawChar(((i*6)+9)+(i*BOXSIZE), 9, i,  ILI9341_BLUE,  ILI9341_WHITE, 2);
  }
  for(i=0; i<=7; i++)
  {
   tft.fillRoundRect(((i*6)+7)+(i*BOXSIZE), 46, BOXSIZE, BOXSIZE, 7, ILI9341_WHITE);
   tft.setCursor(((i*6)+16)+(i*BOXSIZE), 52);
   if(i+8 <= 9){tft.print(i+8);}
   if(i > 9){
   tft.setCursor(((i*6)+7)+(i*BOXSIZE), 46);
   tft.setTextColor(ILI9341_RED);
   tft.setTextSize(3); 
   tft.print(0xA, HEX);}
  }
/*  tft.fillRoundRect(190, 10, BOXSIZE, BOXSIZE, 7, ILI9341_BLACK);
  tft.fillTriangle(200, 35, 220, 35, 210, 20, ILI9341_YELLOW); 
*/  
   // Set which screen is current
  uCurrentScreen = SCREEN_MAIN;

}
//----------------------------------------------------------------------------
//
//----------------------------------------------------------------------------

void SetDisplaySecondScreen(){  

  tft.fillScreen(ILI9341_BLUE);

  tft.fillRoundRect(190, (BOXSIZE*5)+70, BOXSIZE, BOXSIZE, 7,ILI9341_BLACK);
  tft.fillTriangle(215, 270, 195, 270, 204, 290, ILI9341_WHITE);
 tft.fillRoundRect(190, 10, BOXSIZE, BOXSIZE, 7, ILI9341_BLACK);
  tft.fillTriangle(200, 35, 220, 35, 210, 20, ILI9341_YELLOW); 

  uCurrentScreen = SCREEN_SECOND;
}

//----------------------------------------------------------------------------
//
//----------------------------------------------------------------------------
void ProcessTouchSecondScreen(){  

  TS_Point p = ts.getPoint();

  // Serial.print("X = "); Serial.print(p.x);
  //Serial.print("\tY = "); Serial.print(p.y);
  //Serial.print("\tPressure = "); Serial.println(p.z);  


  // Scale from ~0->4000 to tft.width using the calibration #'s
  p.x = map(p.y, TS_MINX, TS_MAXX, 0, tft.width());
  p.y = map(p.x, TS_MINY, TS_MAXY, 0, tft.height());

  if (p.x >= 195 && p.x <= 215) {
    RestoreSPIRegs(SPIR_TFT);
    Serial.print("("); 
    Serial.print(p.x);
    Serial.print(", "); 
    Serial.print(p.y);
    Serial.println(")");
    if (p.y >= 242 && p.y <= 292) {
      SetDisplayMainScreen();
    }
    if (p.y >= 14 && p.y <= 60){
      currentcolor = ILI9341_BLACK;
      tft.fillTriangle(200, 35, 220, 35, 210, 20, ILI9341_CYAN); 
    }
  }
}



//----------------------------------------------------------------------------
//
//----------------------------------------------------------------------------
void ProcessTouchMainScreen(){

  // Retrieve a point  
  TS_Point p = ts.getPoint();
  int i;

  // Serial.print("X = "); Serial.print(p.x);
  //Serial.print("\tY = "); Serial.print(p.y);
  //Serial.print("\tPressure = "); Serial.println(p.z);  


  // Scale from ~0->4000 to tft.width using the calibration #'s
  p.x = map(p.x, TS_MINY, TS_MAXY, 0, tft.width());
  p.y = map(p.y, TS_MINX, TS_MAXX, 0, tft.height());


  Serial.print("(X= "); Serial.print(p.x);
  Serial.print(", Y= "); Serial.print(p.y);
  Serial.println(" )");


  if (p.x >= 20 && p.x <= 55) {
    RestoreSPIRegs(SPIR_TFT);
    oldcolor = currentcolor;

    if (p.y >= 210 && p.y <= 230) { 
      currentcolor = ILI9341_RED; 
      tft.fillRoundRect(7, 7, BOXSIZE, BOXSIZE, 7, ILI9341_YELLOW);
      //tft.fillRoundRect(9, 12, BOXSIZE-4, BOXSIZE-4, 7, ILI9341_RED);
    } 
    else if (p.y >= 180 && p.y <= 200) {
      currentcolor = ILI9341_YELLOW;
      
      tft.fillRoundRect((BOXSIZE+13), 10, BOXSIZE, BOXSIZE, 7, ILI9341_BLACK);
      tft.fillRoundRect((BOXSIZE+15), 12, BOXSIZE-4, BOXSIZE-4, 7, ILI9341_YELLOW);
    } 
    else if (p.y >= 150 && p.y <= 170) {
      currentcolor = ILI9341_GREEN;
      tft.fillRoundRect((BOXSIZE*2)+19, 10, BOXSIZE, BOXSIZE, 7, ILI9341_BLACK);
      tft.fillRoundRect((BOXSIZE*2)+21, 12, BOXSIZE-4, BOXSIZE-4, 7, ILI9341_GREEN);
    } 
    else if (p.y >= 120 && p.y <= 140) {
      currentcolor = ILI9341_CYAN;
      tft.fillRoundRect((BOXSIZE*3)+25, 10,  BOXSIZE, BOXSIZE, 7, ILI9341_BLACK);
      tft.fillRoundRect((BOXSIZE*3)+27, 12,  BOXSIZE-4, BOXSIZE-4, 7, ILI9341_CYAN);
      
    } 
    else if (p.y >= 95 && p.y <= 115) {
      currentcolor = ILI9341_BLUE;
      tft.fillRoundRect((BOXSIZE*4)+31, 10,  BOXSIZE, BOXSIZE, 7, ILI9341_BLACK);
      tft.fillRoundRect((BOXSIZE*4)+33, 12,  BOXSIZE-4, BOXSIZE-4, 7, ILI9341_BLUE);
    } 
    else if (p.y >= 60 && p.y <= 90) {
      currentcolor = ILI9341_MAGENTA;
      tft.fillRoundRect((BOXSIZE*5)+37, 10,  BOXSIZE, BOXSIZE, 7, ILI9341_BLACK);
      tft.fillRoundRect((BOXSIZE*5)+39, 12,  BOXSIZE-4, BOXSIZE-4, 7, ILI9341_MAGENTA);
    }  
    else if (p.y >= 35 && p.y <= 50) {
      currentcolor = ILI9341_RED;
      tft.fillRoundRect((BOXSIZE*6)+43, 10,  BOXSIZE, BOXSIZE, 7, ILI9341_BLACK);
      tft.fillRoundRect((BOXSIZE*6)+45, 12,  BOXSIZE-4, BOXSIZE-4, 7, ILI9341_RED);
    }  
    else if (p.y >= 8 && p.y <= 25) {
      currentcolor = ILI9341_YELLOW;
      tft.fillRoundRect((BOXSIZE*7)+49, 10,  BOXSIZE, BOXSIZE, 7, ILI9341_BLACK);
      tft.fillRoundRect((BOXSIZE*7)+51, 12,  BOXSIZE-4, BOXSIZE-4, 7, ILI9341_YELLOW);
    }  
      }
  else if (p.x >= 70 && p.x <= 100) {
    RestoreSPIRegs(SPIR_TFT);
    oldcolor = currentcolor;if (p.y >= 210 && p.y <= 230) { 
      currentcolor = ILI9341_RED; 
      tft.fillRoundRect(7, 48, BOXSIZE, BOXSIZE, 7, ILI9341_BLACK);
      tft.fillRoundRect(9, 50, BOXSIZE-4, BOXSIZE-4, 7, ILI9341_RED);
    } 
    else if (p.y >= 180 && p.y <= 200) {
      currentcolor = ILI9341_YELLOW;
      
      tft.fillRoundRect((BOXSIZE+13), 48, BOXSIZE, BOXSIZE, 7, ILI9341_BLACK);
      tft.fillRoundRect((BOXSIZE+15), 50, BOXSIZE-4, BOXSIZE-4, 7, ILI9341_YELLOW);
    } 
    else if (p.y >= 150 && p.y <= 170) {
      currentcolor = ILI9341_GREEN;
      tft.fillRoundRect((BOXSIZE*2)+19, 48, BOXSIZE, BOXSIZE, 7, ILI9341_BLACK);
      tft.fillRoundRect((BOXSIZE*2)+21, 50, BOXSIZE-4, BOXSIZE-4, 7, ILI9341_GREEN);
    } 
    else if (p.y >= 120 && p.y <= 140) {
      currentcolor = ILI9341_CYAN;
      tft.fillRoundRect((BOXSIZE*3)+25, 48,  BOXSIZE, BOXSIZE, 7, ILI9341_BLACK);
      tft.fillRoundRect((BOXSIZE*3)+27, 50,  BOXSIZE-4, BOXSIZE-4, 7, ILI9341_CYAN);
      
    } 
    else if (p.y >= 95 && p.y <= 115) {
      currentcolor = ILI9341_BLUE;
      tft.fillRoundRect((BOXSIZE*4)+31, 48,  BOXSIZE, BOXSIZE, 7, ILI9341_BLACK);
      tft.fillRoundRect((BOXSIZE*4)+33, 50,  BOXSIZE-4, BOXSIZE-4, 7, ILI9341_BLUE);
    } 
    else if (p.y >= 60 && p.y <= 90) {
      currentcolor = ILI9341_MAGENTA;
      tft.fillRoundRect((BOXSIZE*5)+37, 48,  BOXSIZE, BOXSIZE, 7, ILI9341_BLACK);
      tft.fillRoundRect((BOXSIZE*5)+39, 50,  BOXSIZE-4, BOXSIZE-4, 7, ILI9341_MAGENTA);
    }  
    else if (p.y >= 35 && p.y <= 50) {
      currentcolor = ILI9341_RED;
      tft.fillRoundRect((BOXSIZE*6)+43, 48,  BOXSIZE, BOXSIZE, 7, ILI9341_BLACK);
      tft.fillRoundRect((BOXSIZE*6)+45, 50,  BOXSIZE-4, BOXSIZE-4, 7, ILI9341_RED);
    }  
    else if (p.y >= 8 && p.y <= 25) {
      currentcolor = ILI9341_YELLOW;
      tft.fillRoundRect((BOXSIZE*7)+49, 48,  BOXSIZE, BOXSIZE, 7, ILI9341_BLACK);
      tft.fillRoundRect((BOXSIZE*7)+51, 50,  BOXSIZE-4, BOXSIZE-4, 7, ILI9341_YELLOW);
    }  
  }
  else if (p.x >= 195 && p.x <= 215) {
    RestoreSPIRegs(SPIR_TFT);
    oldcolor = currentcolor;
    if (p.y >= 14 && p.y <= 60) {
      currentcolor = ILI9341_BLACK;
      SetDisplaySecondScreen();
      return;
    }
  }

 if(p.x >= 20 && p.x <= 55)
 {
  if (oldcolor != currentcolor) 
  {
      RestoreSPIRegs(SPIR_TFT);
   
 //      for(i=0; i<=7; i++)
//  {
//   tft.fillRoundRect(((i*6)+7)+(i*BOXSIZE), 7, BOXSIZE, BOXSIZE, 7, ILI9341_WHITE);
//  }
//  for(i=0; i<=7; i++)
//  {
//   tft.fillRoundRect(((i*6)+7)+(i*BOXSIZE), 46, BOXSIZE, BOXSIZE, 7, ILI9341_WHITE);
//  }
     if (oldcolor == ILI9341_RED) 
        tft.fillRoundRect(7, 10, BOXSIZE, BOXSIZE, 7, ILI9341_RED);
      if (oldcolor == ILI9341_YELLOW) 
        tft.fillRoundRect((BOXSIZE+13), 10, BOXSIZE, BOXSIZE, 7, ILI9341_YELLOW);
      if (oldcolor == ILI9341_GREEN) 
        tft.fillRoundRect((BOXSIZE*2)+19, 10, BOXSIZE, BOXSIZE, 7, ILI9341_GREEN);
      if (oldcolor == ILI9341_CYAN) 
        tft.fillRoundRect((BOXSIZE*3)+25, 10,  BOXSIZE, BOXSIZE, 7, ILI9341_CYAN);
      if (oldcolor == ILI9341_BLUE) 
        tft.fillRoundRect((BOXSIZE*4)+31, 10,  BOXSIZE, BOXSIZE, 7, ILI9341_BLUE);
      if (oldcolor == ILI9341_MAGENTA) 
        tft.fillRoundRect((BOXSIZE*5)+37, 10,  BOXSIZE, BOXSIZE, 7, ILI9341_MAGENTA);
      if (oldcolor == ILI9341_RED)  
        tft.fillRoundRect((BOXSIZE*6)+43, 10, BOXSIZE, BOXSIZE, 7, ILI9341_RED);
      if (oldcolor == ILI9341_YELLOW)  
        tft.fillRoundRect((BOXSIZE*7)+49, 10, BOXSIZE, BOXSIZE, 7, ILI9341_YELLOW);
      } 
    }
  else if(p.x >= 70 && p.x <= 100) 
  {
    if (oldcolor != currentcolor)
    {
      RestoreSPIRegs(SPIR_TFT);
      if (oldcolor == ILI9341_RED) 
        tft.fillRoundRect(7, 48, BOXSIZE, BOXSIZE, 7, ILI9341_RED);
      if (oldcolor == ILI9341_YELLOW) 
        tft.fillRoundRect((BOXSIZE+13), 48, BOXSIZE, BOXSIZE, 7, ILI9341_YELLOW);
      if (oldcolor == ILI9341_GREEN) 
        tft.fillRoundRect((BOXSIZE*2)+19, 48, BOXSIZE, BOXSIZE, 7, ILI9341_GREEN);
      if (oldcolor == ILI9341_CYAN) 
        tft.fillRoundRect((BOXSIZE*3)+25, 48,  BOXSIZE, BOXSIZE, 7, ILI9341_CYAN);
      if (oldcolor == ILI9341_BLUE) 
        tft.fillRoundRect((BOXSIZE*4)+31, 48,  BOXSIZE, BOXSIZE, 7, ILI9341_BLUE);
      if (oldcolor == ILI9341_MAGENTA) 
        tft.fillRoundRect((BOXSIZE*5)+37, 48,  BOXSIZE, BOXSIZE, 7, ILI9341_MAGENTA);
      if (oldcolor == ILI9341_RED)  
        tft.fillRoundRect((BOXSIZE*6)+43, 48, BOXSIZE, BOXSIZE, 7, ILI9341_RED);
      if (oldcolor == ILI9341_YELLOW)  
        tft.fillRoundRect((BOXSIZE*7)+49, 48, BOXSIZE, BOXSIZE, 7, ILI9341_YELLOW);
     }
  }
}




